<?php

namespace FiloBlu\Refilo\Plugins\Mview;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Mview\View\Changelog as MagentoChangelog;
use Magento\Framework\Mview\View\ChangelogTableNotExistsException;
use Magento\Framework\Phrase;

class Changelog
{
    /**
     * @var AdapterInterface
     */
    private $connection;

    /**
     * @param ResourceConnection $connection
     */
    public function __construct(ResourceConnection $connection){

        $this->connection = $connection->getConnection();

    }


    public function aroundGetList(MagentoChangelog $subject , callable $proceed, $fromVersionId, $toVersionId){
        if($subject->getName()!=="beehive_stock_bridge_cl"){
            return $proceed($fromVersionId,$toVersionId);
        }
        //cerca i changelog nella colonna corretta ovvero sku invece che sulla standard entity_id
        $changelogTableName = $this->connection->getTableName($subject->getName());
        if (!$this->connection->isTableExists($changelogTableName)) {
            throw new ChangelogTableNotExistsException(new Phrase("Table %1 does not exist", [$changelogTableName]));
        }

        $select = $this->connection->select()->distinct(
            true
        )->from(
            $changelogTableName,
            ["sku"]
        )->where(
            'version_id > ?',
            (int)$fromVersionId
        )->where(
            'version_id <= ?',
            (int)$toVersionId
        );
        return $this->connection->fetchCol($select);
    }

    public function aroundCreate(MagentoChangelog $subject , callable $proceed)
    {
        //crea la tabella beehive_stock_bridge_cl con la colonna varchar(255) sku invece che di tipo int
        if($subject->getName()!=="beehive_stock_bridge_cl"){
            return $proceed();
        }

        $changelogTableName = $this->connection->getTableName($subject->getName());
        if (!$this->connection->isTableExists($changelogTableName)) {
            $table = $this->connection->newTable(
                $changelogTableName
            )->addColumn(
                'version_id',
                Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Version ID'
            )->addColumn(
                "sku",
                Table::TYPE_TEXT,
                255,
                ['unsigned' => true, 'nullable' => false],
                'Sku'
            );
            $this->connection->createTable($table);
        }
    }

}
